//____________________________________________________________________________________________________________
//  ce programme illustre l'utilisaion de la classe QSerialPort pour gérer une liaison série
//  il s'agit de récupérer les deux coordonnées x, y et l'état du boutton d'un joystick série
//  le joystick envoie une trame de la forme xx yy bb avec xx et yy les coordonnées et bb l'état du boutton.
// exemle: 125 45 1
//
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QDebug>
#include <QPainter>
//_________________________________   constructeur  ___________________________________________________
MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    //    le titre de la fenêtre
    this->setWindowTitle("Serial communications");

    serial = new QSerialPort("/dev/ttyUSB0", this);     // le port de communication série
    serial->open(QIODevice::ReadWrite);                 // ouverture du port en lecture/ecriture
    serial->setDataBits(QSerialPort::Data8);            // 8 bits data
    serial->setBaudRate(QSerialPort::Baud115200);       // baud rate
    serial->setFlowControl(QSerialPort::NoFlowControl); // pas de control de flux
    serial->setParity(QSerialPort::NoParity);           // pas de parité
    serial->setStopBits(QSerialPort::OneStop);          // 1 stop bit

    // on connecte le signal readyRead (données arrivées) du port à la méthode data2read de moi (this)
    connect(serial, SIGNAL(readyRead()), this, SLOT(data2read()));
}
//_________________________________   destruction   ____________________________________________________
MainWindow::~MainWindow()
{
    delete ui;
    if (serial)delete serial;
}
//______________________________________________________________________________________________________
//   méthode activée quand des données arrivent sur le port de comm
void MainWindow::data2read(){
    //qDebug()<<serial->bytesAvailable();
    QByteArray data;                // un tableau d'octets
    data = serial->readLine();      // lecture d'une trame décrivant la position du joystick
                                    // sous la forme  xxx yyy bbb

    QString str(data);                              // transformation en QString
    QStringList values = str.split(" ");            // séparation des arguments, séparateur espace
    //qDebug()<<data;
    if (values.size()<3) return;                    // la trame est incomplète

    // pour dessiner un cercle sur la fenêtre
    int w = this->centralWidget()->width();
    int h = this->centralWidget()->height();

    // la position du joystick réajustée à la taille de la fenètre
    pos.setX(values[0].toInt()/512.*w/2+w/2);
    pos.setY(-values[1].toInt()/512.*h/2+h/2);

    joystickButton = values[2]=="1\r\n";    // dans notre exemple, la liaison série reçoit les caractères de fin de ligne
    // appel de la méthode paintEvent
    repaint();

}
//___________________________________________________________________________________________________________
void MainWindow::paintEvent(QPaintEvent *evt){
    QPainter painter(this);
    // dessin d'un cercle à la position du joystick
    painter.setBrush(joystickButton ? QBrush(Qt::red) : QBrush(Qt::white));
    painter.drawEllipse(pos, 20, 20);

}
